function atmosrd,file,plot=plot,xran=xran,tran=tran
;+
;   atmosrd,file
;
;            reads a multi atmos file
;
;   Modified 14-Dec-1992 by P. Judge
;   dptype is examined:
;   dptype= 'height'  then variable height is set
;   dptype= 'cmass'  then variable cmass is set
;   Modifications:  P. Judge Aug 5 1993
;     approximate calculations of height scale added.
;   Modifications:  P. Judge Aug 24 1993
;     plot keyword parameter added
;     plot keyword parameter added
;     plot keyword parameter added
;   
;-
@common

if(n_params(0) lt 1) then begin
  print,'atmosrd,file'
  return,-1
endif
;
;
text=''
openr,lu,file,/get_lun
openw,lu2,'dums',/get_lun
while (not eof(lu)) do begin
  readf,lu,text,format='(a)'
  if(strmid(text,0,1) ne '*') then printf,lu2,text,format='(a)'
endwhile
free_lun,lu
close,lu2
openr,lu2,'dums'
atmoid=''
dptype=''
readf,lu2,atmoid
print,atmoid
readf,lu2,dptype
dptype=strtrim(dptype,2)
readf,lu2,gravlg
grav=10.^gravlg
ndep=0L
readf,lu2,ndep
tab=dblarr(5,ndep)
readf,lu2,tab
tab=transpose(tab)
temp=tab(*,1)
nne=tab(*,2)
vel=tab(*,3)
vturb=tab(*,4)
case strupcase(strmid(dptype,0,1)) of
  'M': begin
     cmasslg=tab(*,0)
     cmass=10.^cmasslg
   end
   'H': height=tab(*,0)
   else: print,' Depth scale type ', dptype, '  unknown'
endcase
on_ioerror,end_read
nh=dblarr(6,ndep)
readf,lu2,nh
;
xx=dblarr(6)*0.+1.
rho=xx#nh*1.3625*param.umass
if (strupcase(strmid(dptype,0,1)) eq 'H') then begin
  ;  trapezoidal integration of height for cmass
  print,'atmosrd:  computing cmass scale from height '
  cmass=height*0.
  heightt=height
  cmass(0)= ((nh(0)+nh(5))*1.103+nne)*param.bconst*temp(0)/grav ;hse top point
  for k=1,n_elements(height)-1 do $
    cmass(k) = cmass(k-1)+ 0.5*(height(k-1)-height(k))*(rho(k)+rho(k-1)); *1.e5
endif else if (strupcase(strmid(dptype,0,1)) eq 'M') then begin
   ;  dz = dm / rho
   print,'atmosrd:  computing height scale from cmass '
   height=cmass*0.
   for i=ndep-2,0,-1 do begin 
      dm = cmass(i+1)-cmass(i)
      rhom=(rho(i+1)+rho(i))/2.
      dz=dm/rhom
      height(i) = height(i+1)+dz
   endfor   
endif 
end_read:
free_lun,lu2
cmass=reform(cmass)
temp=reform(temp)
rho=reform(rho)
nne=reform(nne)
vel=reform(vel)
vturb=reform(vturb)
atmos={atmoid:atmoid,grav:grav,cmass:cmass,height:height,temp:temp,nh:nh,nne:nne,vturb:vturb,vel:vel,rho:rho}

if(n_elements(plot) ne 0) then begin
  !p.multi=[0,0,2,0,0]
  if(n_elements(xran) eq 0) then xran=[0,0]
  if(n_elements(tran) eq 0) then tran=[0,0]
  plot_oi,cmass,temp,xtit='cmass',ytit='Temp [K]',xran=xran,yran=tran,ysty=1,$
   titl=atmoid
  plot_oo,cmass,nne,xtit='cmass',ytit='N!Le!N [cm!U-3!N]',xran=xran
  oplot,cmass,nh(0,*),lines=1
  oplot,cmass,nh(5,*),lines=2
  legend,lines = indgen(3),['Ne','NH(n=1)','Np']
endif

return,atmos
end

