function fcn, v,alpha,deriv
;+
; non-linear function that appears ahead of the second deriv term,
; in crank-nicolson scheme
;-
case deriv of
   0: return,v^(alpha/(alpha+1.d0))
   1: return,alpha/(alpha+1.d0) * v^(-1./(alpha+1.d0))
endcase
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro cranknic,x,u,dt
;
;+
;  Crank Nicolson integration of diffusive part of the 
;  operator, energy equation only, uniform grid only
;- 
@common
b=size(u)
t=reform ( (pt(u))[1,*]  )
told=t
n=total(u[3:b[1]-1,*],1) + nelec(u)
;
; grid assumed uniform
;
dx=x(1)-x(0)
;
;  Conduction part of the energy equation 
;  is  dT/dt = f(T)  d^2T / dz^2
;
;  f(T) is non-linear
;  f(T) = 2/3 Kappa/nk T^5/2, or 
;  f(T) = 2/3  param.kappa0/n/param.bconst * T^param.alpha
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;  define C such that
;  f(T) = c T^param.alpha
;
a=param.alpha
ap1=a+1.d0
v=T^ap1  ; variable to be solved by tridiagonal system
vprev=v
;
const= 2./3.*param.kappa0/n/param.bconst
cconst= - dt/dx/dx/2. * const
r=-dt/dx/dx/2. * const*v^(a/ap1)
;
;  use old value of T in f(T) as a first guess
;  upper, lower, diagnonal part of tri diagonal matrix
;
aoff=-r
adiag=(1.+2.*r)
rhs= r*der2(v) + v
v=trisol(aoff,adiag,aoff,rhs)*1.05

;goto, nonr  ; Don't do an implicit component
;
; print out the error from not doing NR
;
err= v -vprev  - r* (der2(vprev) + der2(v))
w=.5d0 ; semi-implicit is w=0.5
rp=-dt/dx/dx/2. * const*v^(a/(a+1.))
d2o=der2(vprev)
d2= der2(v)
;
err= 1.d0 - vprev/v  - (rp*w + r*(1.-w))*(d2+d2o)/v
ok=where(t gt 1.d3)
mx=max(abs(err(ok)))
;
;  Newton Raphson part.  We solve d/ dt v  = f(v)  d^2/dx^2 v
;   with f(v) evaluated  as w1*f(v^n+1) + w0*f(v^n)
;   the above correction v-vold is the estimate with w = 0.
; 
count=0
;
nr:
while (mx gt 1.d-4) do begin 
   vsave=v
   d2o=der2(vprev)
   d2= der2(v)
   xx=  - cconst*(w*fcn(v,a,0)+(1.-w)*fcn(vprev,a,0))
   err=v-vprev + xx*(d2+d2o)

; set up tridiagonal NR matrix

   der=fcn(v,a,1)
   diag= 1.d0  -  cconst* w*der*(d2+d2o)   - 2.*xx

; solve

   h=trisol(xx,diag,xx,-err)
   mx=max(abs(h/v))
   v=v + h  ; next estimate
   count=count+1
;   print,'iteration ',count,' max = ',mx
   goto, nr
endwhile
;
nonr:
t=v^(1./ap1)
u(2,*)=u(2,*) +param.bconst*3./2.*(t-told)*n  ; update vector
return
end
