@common
@inpt
param=add_tag(param,atomn(param.elements,/num),'ielements')
;
desc=states()
ysize= (200  > n_elements(desc)*40) < 800
window,13,xsize=400,ysize=ysize,title='Gas dynamic changes'
window,14,xsize=1000,ysize=580,title='Gas dynamic evolution'
window,1
told=0.
dt=1.d-5
;
; initial conditions in u, sets time t and time index i
;
initial,x,u ,atmos='atmos.val3c',i,t
;
; plot initial state
;
uplot,x,u,0.,/ra,el='c'
;
uold=u
a=size(u) & ny=a(1) & nx=a(2)
xr=[param.xmin,param.xmax]/1.d8+[-1,1]
;
bcount=0
;
noutput=100 ; print/plot changes every noutput iterations
deltatold=-1.
;
stime=systime(1)
okx=2+indgen(nx-3)
count=0
sl=param.slowdown
;
WHILE t LT param.tmax DO BEGIN 
   str=strcompress(string(i),/rem)
   timestep,x,uold,u,dt,cfl
   uold=u
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; integrate up in time
;
   tvdlf,x,u,dt,t ;,pdelta=pdelta
;   maccormack,x,u,dt,t,pdelta=pdelta
   if(param.osplit ne 0) then cranknic,x,u,dt
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  non TVD/ NaN problems
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   du= total ( (abs(u-shift(u,[0,1])))[*,okx] )
   duold= total ( (abs(uold-shift(uold,[0,1])))[*,okx] )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; problem cases
   k=where(finite(u) eq 0,ninf) 
   utest=u(2:*,*)
   kneg=where(utest lt -1.d-10,nneg)
   u(2:*,*) = u(2:*,*) > 0.
   if(ninf gt 0 or (du/duold-1) gt 1.d-1) then begin 
      print,'negatives, slower',ninf
      param.slowdown= (1.05*param.slowdown < 5.d2) 
      u=uold
      bcount=bcount+1
      if(bcount mod 20 eq 0) then begin 
         param.slowdown *=1.2
         print,' nneg ',nneg,'n_inf',ninf,'du/duold-1',du/duold-1.,$
               form='(2(a,i7,1x),a,1x,e10.3)'
         print,utest(kneg)
         bcount=0
         goto, skippy
      endif
   end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   param.slowdown=(param.slowdown/1.03) > 1.  
;   
; stop at steady state
;
   machmx=max(abs(u(1,*)/u(0,*))/csound(u))
   if(t gt 1.d0 and machmx lt 1.d-3) then message,'Mach no < 1.e-3, steady state reached'
;
; advance time 
;
   t = t+dt 
   i = i+1l                     ;
   it=long(nint(t))
   deltat= t - it
   if(deltat ge 0. and deltatold lt 0.) then begin 
      str=strcompress(string(it),/remove_all)
      file='u'+str+'.sav'
      time=t
      if(it mod 10 eq 0) then print,'saving ',file
      if(str eq '0') then save,file=file,param,it,time,x,u else save,file=file,it,time,x,u
      save,file='restart.sav',it,time,x,u
   endif
   deltatold=deltat
   if((i-1) mod (noutput*30l) eq 0) then print,'     n       t[s]     cfl      dt       max du      t cpu / t sun'
   if(i mod noutput eq 0) then begin 
      uplot,x,u,t,el='c',/ra
      delta=changes(u,uold,/plot)
      dtt=(systime(1)-stime)/(t-told)
      dflag= ' ' & pflag=' '
      if(delta gt 1.) then dflag='*'
;      if(pdelta gt 1.) then pflag='*'
      print,i,t,cfl,dt,delta,dflag,dtt,form='(i8,1x,f8.2,2(1x,e9.1),1x,e9.1,a,(1x,e9.1))'
      stime=systime(1)
      told=t
      bcount=0
   endif
   skippy:
   sl+=param.slowdown
   count +=1l
   avgsl=sl/count
   if((i mod 1000) eq 0) then print, 'Mean slowdown =',avgsl
ENDWHILE
print,'Mean slowdown =',avgsl

;
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'transp.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
