pro figs, atoms,ps=ps,tmax=tmax,xmax=xmax,length=length,other_footpoint=other_footpoint,$
          equilibrium=equilibrium
@common

restore,'u0.sav'

if(n_elements(length) eq 0) then length=20

if(n_elements(atoms) eq 0) then atoms=['he','c','o','si']
if(n_elements(tmax) eq 0) then tmax=150
if(n_elements(xmax) eq 0) then xmax=80.

x0=0
x11=3.0*0.725d8  ; 2 arcsec
o=''
if(n_elements(other_footpoint) ne 0) then begin 
   o='o'
endif

estr=''
ieq=0
if(n_elements(equilibrium)  ne 0) then ieq=1
if(ieq eq 1) then estr='e'

if(n_elements(ps) eq 0) then begin 

   loadct,1
;   gamma_ct,1
   window,0,xsize=1400,ysize=1000,ret=2
   size=1.5

endif else begin 

   loadct,0
;   gamma_ct,1
   ps,/col,/encaps,/cop,'profiles'+o+estr+'.ps'
   device,xsize=12,ysize=9,/inches
   publish
   size=1.4
endelse
!p.charsize=size

panel,0,0,nx=6,ny=n_elements(atoms),order=0,ydist=0,xdist=1,xlabel=0,ylabel=0

for m=0,n_elements(atoms)-1 do begin 
      ss=atomn(atoms(m),/str)
      case(atoms(m)) of
         'he': begin 
            lambdas =[584,537,10833,304,256,1640]
            ions=[1,1,1,2,2,2]
            mx=2.
         end
         'c': begin 
            lambdas=[1335,977,1548,312,41,34]
            ions=[2,3,4,4,5,6]
            mn=0
         end
         'o': begin 
            lambdas=[833,508,788,1032,22,19]
            ions=[2,3,5,6,7,8]
            mn=0
         end
         'si': begin 
            lambdas=[1526,1206,1393,458,63,621] ; last are Si VIII, Si X
            lambdas=[1526,1206,1393,458,275,621] ; last are Si VII, Si X
            ions=[2,3,4,4,7,10]
            mn=0
         end
      endcase
      lambdas=float(lambdas)

      for i=0,n_elements(lambdas)-1 do begin 
         lrange=lambdas(i)+[-2,2]
         if(ieq eq 1) then postproc,ll,tmax=tmax,el=atomn(atoms(m)),sp=ions(i),/eq,lrange=lrange else $
            postproc,ll,tmax=tmax,el=atomn(atoms(m)),sp=ions(i),lrange=lrange
         j=where(ll.lambda gt lambdas(i)-2. and ll.lambda lt lambdas(i)+2.) 
         mx=max(ll(j).omega)
         j=j(!c)                 ;take just the strongest one
         if(i eq 0) then l=ll(j) else l=[l,ll(j)]
      endfor
      postprof,l
;
; Plot profiles 
;  
;      print,x11,x0
      for i=0,n_elements(l)-1 do begin
         x1=x11
         if(n_elements(other_footpoint) ne 0) then begin 
            x0=max(l(i).synth.pos)-x11
            x1=max(l(i).synth.pos)
         endif
         iz=where(l(i).synth.pos ge x0 and l(i).synth.pos le x1,nz)
         im=total ( reform(l[i].synth.i[*,iz,*]),2)/nz ; average
         im=im>0
         dopp=l[i].synth.dopp
         xr=where(abs(dopp/1.e5) lt xmax)
         dopp=dopp(xr)
         im=im(xr,*)
         origin=[dopp[0],0]/1.e5
         sc=[l[i].synth.dopp[1]-l[i].synth.dopp[0],1.e5]/1.e5
;
; total integrated line emission, erg/cm2/s/sr
         ntim=(size(im))(2)
         imx=fltarr(ntim)
         for ij=0,ntim-1 do imx(ij)=int_tabulated(dopp,im(*,ij))
         mx=max(imx)
         title=atomn(l[i].el)+' '+roman(l[i].sp)+' '
         title1=strtrim(string(fix(l[i].lambda)),2)+angstrom()
         title2= strtrim(string(mx,form='(E8.1)'),2)
         if(ieq eq 1) then title +=' IE'
         xtitle='' & ytitle=''
         lab=(m eq n_elements(atoms)-1 and i eq 0 )
         if(lab) then begin
            xtitle='Doppler shift km/s'
            ytitle='Time s'
         endif
;         if(n_elements(other_footpoint) ne 0) then im=reverse(im)  ; swap sign of doppler shift, along LOS
         plot_image,max(im)-im,origin=origin,sc=sc,title='',/nosq,xtitle=xtitle,ytitle=ytitle
         x=!x.crange(0)+.05*(!x.crange(1)-!x.crange(0))
         y=!y.crange(0)+.9*(!y.crange(1)-!y.crange(0))
         y1=!y.crange(0)+.78*(!y.crange(1)-!y.crange(0))
         y2=!y.crange(0)+.66*(!y.crange(1)-!y.crange(0))
         xyouts,x,y,title,size=size/1.5
         xyouts,x,y1,title1,size=size/1.5
         xyouts,x,y2,title2,size=size/1.5
         oplot,[0,0],!y.crange,lines=0,col=!d.n_colors*.25
         for jjj=0,2 do oplot,!x.crange,[10,10]+length*jjj,lines=1,col=!d.n_colors*.5
         lab= (m eq n_elements(atoms)-2 and i eq n_elements(l)-1)
         panel,/next,xlabel=lab,ylabel=lab
      endfor
; stop
      xtra=''
;
; plot moments of lines
;
      if(ieq eq 1) then xxtra='e' else xxtra=''
      if(n_elements(ps) eq 0) then begin
         mmts,l,/plot
         write_png,ss+xtra+'_mmts'+xxtra+'.png',tvrd(/true)
      endif
;   endfor                       ; eqm or not
endfor                          ; atoms

if(n_elements(ps) ne 0) then begin 

   publish,/res
   psclose

endif

return
end
