pro mmts,lines,plot=plot
;+
; take moments of line profiles
; 0 = intensity
; 1 = Doppler shift (km/s)
; 2 = width (km/s)
; the structure lines is modified and the tag lines.synth.moment is appended.
;-
;
nl=n_elements(lines)
a=size(lines[0].synth.i)
;
moment=fltarr(3,a(2),a(3))
syn=lines.synth
;
; update the sub-structure
;
syn=rem_tag(syn,'moment')
syn=add_tag(syn,moment,'moment')
lines=rem_tag(lines,'synth')
lines=add_tag(lines,syn,'synth')
;
dy=(syn(0).pos(1)-syn(0).pos(0))/1.e8
d=syn(0).dopp
dd=deriv(d)
for il=0,nl-1 do begin
   for iy=0,a(2)-1 do begin
      x= reform( lines[il].synth.i[*,iy,*] )  ; wavelength vs time for each line and position
      lines[il].synth.moment(0,iy,*) = dd        # x
      lines[il].synth.moment(1,iy,*) =  (d*dd)        # x 
      lines[il].synth.moment(2,iy,*) = (d*d*dd)     # x 
   endfor
endfor
lines.synth.moment[1,*,*]=lines.synth.moment[1,*,*,*]/lines.synth.moment[0,*,*,*]/1.e5
lines.synth.moment[2,*,*]=sqrt(lines.synth.moment[2,*,*]/lines.synth.moment[0,*,*])/1.e5

if(n_elements(plot) ne 0) then begin 
   !p.multi=[0,nl+1,3,0,0]
   !p.charsize=1.6
   !x.margin=[8,1]
   !y.margin=[4,2]
   titl=['Intensity','Doppler shift km/s','Width km/s']
   iz=a(3)
   iz=25                ; subset of positions on sun to plot, hard to see otherwise
   it=a(3)-1
;   it=180
   imx=max(lines.synth.moment(0,0:iz,0:it))
   mn=[0,-100,0]
   mx=[imx,100,200]
   for ii=0,2 do begin
      for il=0,nl-1 do begin
         ist=roman((lines[il].sp))
         title=strcompress(atomn(lines[il].el,/str)) +' '+ist+' '+string(lines[il].lambda,format='(f11.1)')+' '+angstrom()
         print,title
         if(il eq 0) then tit=titl(ii) else tit=''
         xtitle='' & ytitle=''
         if(ii eq 2 and il eq 0) then begin 
            xtitle='X Mm'
            ytitle='time s'
         endif
         if(ii eq 0) then begin 
            ratio=imx/max(lines[il].synth.moment(0,0:iz,0:it))
            plot_image,/nosq,reform(lines[il].synth.moment(ii,0:iz,0:it)),tit=tit,min=mn(ii),max=mx(ii)/ratio,scale=[dy,1]
            xyouts,1,it*.9,/data,strcompress(title),charsize=0.9
            xyouts,1,it*.8,/data,strcompress('x'+string(ratio,format='(f7.1)')),charsize=.8
         endif else begin
            ratio=1.
            plot_image,/nosq,reform(lines[il].synth.moment(ii,0:iz,0:it)),min=mn(ii),max=mx(ii)/ratio,tit=tit,xtitle=xtitle,ytitle=ytitle,scale=[dy,1]
         endelse
         if(il eq (nl-1)) then begin 
            plot,indgen(2),/nodata,xsty=31,ysty=31
            cbar,findgen(5,5),min=mn(ii),max=mx(ii),$
                 0.1,0.3,0.1,0.8,ticklen=-0.5,col=(!d.n_colors-1),$
                 xcharsize=.8,title=titl(ii),/right
         endif
      endfor
   endfor
endif

!p.multi=0

return
end
