;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro tvdlf,x,u,dt,t,pdelta=pdelta
@common
;   
; TVDLF method
;
;  Toth section 4.2
;
a=size(u)
ny=a(1) & nx=a(2)
;
; assume uniform grid
;
dx=x(1)-x(0)
;
; delta ubar 4.16 of Toth
;
dub=delubar(u,limiter=param.limiter)
sh=[0,1]
;
case param.tvorder of
   1: begin ; first order in space and time.  
      left=u
      right=shift(u,-sh)
      fp= (flux(x,left)+flux(x,right))*.5d0
      fm= shift(fp,sh)
      use=u
   end
   -2:begin ; first order in time, second in space, 
;
; second order spatial accuracy in space, eq. 4.8, set sec=1.0
;
      left=u+.5*dub
      right=shift(u-0.5*dub,-sh)
      fp= (flux(x,left)+flux(x,right))*.5d0
      fm= shift(fp,sh)
      use=u
   end
   2: begin ; second order in space and time
;
;  TVDLF loop, if changes big then try again w much smaller dt
;    which is guaranteed to make a small correction.  This is essentially
;    just a time step limiter to ensure the preddictor/corrector 
;    corrections are small. 
;
; predictor step eq. 4.11
;
      dd=(flux(x,u+0.5*dub)-flux(x,u-0.5*dub))/dx -(source(u) + xtra)
      unphalf= u  - 0.5 * dt * dd 
;
; check change magnitudes, reduce dt as needed
;
      pdelta=changes(unphalf,u)          ; actual changes 
      ratio= (1./pdelta)/param.slowdown < 1.d0 ; allowed/actual
      dt = (( dt*ratio ) > param.tsmall) < .1d0
      if(dt le 0.d0) then message,'dt too small'
;
; proceed with smaller step
;
      unphalf=u -0.5* dt * dd
;
;  linear extrapolants, eqs 4.12, must use 0.5*dx/dxav
;
      left=unphalf + 0.5*dub 
      right=shift(unphalf-0.5*dub,-sh)
;
; fluxes at cell interfaces, eqns. 4.9 4.10
;
      fp= (flux(x,left)+flux(x,right))*.5d0
      fm=shift(fp,sh)
      use=unphalf
   end
endcase
;
av=(use+shift(use,-sh))/2.
xtra=call_function(param.driver,use,x,t)
;
; propagation speed, arithmetic mean
;
cmax=u*0.
c=csound(av) ;+ abs(av(1,*)/av(0,*))
; orig for i=0,ny-1 do cmax(i,*)=c
for i=1,2 do cmax(i,*)=c   ; p, E
cmax(0,*)=abs(av(1,*)/av(0,*))
for i=3,ny-1 do cmax(i,*)=cmax(0,*)   
; 
phip=dt/dx*cmax*(right-left)
phim=shift(phip,sh)
;
;corrector  eq. 4.3
;
new=u -dt/dx*(fp-fm) + 0.5*(phip-phim) + (source(use)+xtra)*dt ; explicit case
;
bound,x,new
;
u=new
return 
end
