;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro tvdlfis,x,u,dt,t,pdelta=pdelta
@common
;   
; TVDLF method
; 1 Ghost cell at either end of the grid
; implicit source term in full corrector step
;
;  Toth section 4.2
;
message,'does not work yet, does not conserve particles'
a=size(u)
ny=a(1) & nx=a(2)
;
; ghost cells at end
;
j=1+indgen(nx-3)
uold=u
;
; uniform grid
;
dx=x(1)-x(0)
;
; delta ubar 4.16 of Toth
;
dub=delubar(u,limiter=param.limiter)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  TVDLF loop, if changes big then try again w much smaller dt
;    which is guaranteed to make a small correction.  This is essentially
;    just a time step limiter to ensure the predictor/corrector 
;    corrections are small. 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Predictor step eq. 4.11
;
xtra=call_function(param.driver,u,x,t)
dd=(flux(x,u+0.5*dub)-flux(x,u-0.5*dub))/dx -(source(u) + xtra)
pred= u  - 0.5 * dt * dd 
;
; check magnitudes, reduce dt as needed
;
pdelta=changes(pred,u)  ; actual changes 
;
ratio= (1./pdelta)/param.slowdown < 1.d0  ; allowed/actual
dt = (( dt*ratio ) > param.tsmall) < .1d0
;
; proceed with smaller step
;
pred=u -0.5* dt * dd
;bound,x,pred
if(dt le 0.d0) then message,'dt too small'
;
;  linear extrapolants, eqs 4.12, must use 0.5*dx/dxav
;
leftp=pred + 0.5*dub ;OK
ritem=pred - 0.5*dub
ritep=shift(pred,[0,-1]) - 0.5*shift(dub,[0,-1])
leftm=shift(pred,[0,1]) + 0.5*shift(dub,[0,1])
;
; fluxes at cell interfaces, eqns. 4.9 4.10
;
fp= (flux(x,leftp)+flux(x,ritep))*.5d0
fm= (flux(x,leftm)+flux(x,ritem))*.5d0
;
; propagation speed, arithmetic mean
;
av=(leftp+ritep)/2.
;
; geometric
;av=sqrt(leftp*ritep)
cmax=u*0.
c=csound(av) + abs(av(1,*)/av(0,*))
;for i=0,ny-1 do cmax(i,2:nx-3)=c(2:nx-3)
for i=0,ny-1 do cmax(i,*)=c
phip=dt/dx*cmax*(ritep-leftp)
phim=dt/dx*shift(cmax,[0,1])*(ritem-leftm)
;
;corrector  eq. 4.3
;
corre=- dt/dx*(fp-fm) + 0.5*(phip-phim) 
;
; implicit or explicit
;
xtra=call_function(param.driver,pred,x,t)
ss=source(pred)
new=u + corre ; advection, explicit part
;
;
if(t lt 1) then begin 
   new=new+ss*dt ; explicit
endif else begin 
;
; implicit source term
;
ja=sjacobian(pred)  ; numerical jacobian, ja(ny,ny,nx)
;
unit=dblarr(ny,ny)
unit(indgen(ny),indgen(ny))=1.d0
for ix=1,nx-2 do begin 
   mat=((unit - reform(ja(*,*,ix))*dt))
   rhs=reform(ss(*,ix)*dt)
   print
   print,'Mat'
   print,mat
   print,'rhs'
   print,rhs
   lud,mat
   lub,mat,rhs
   print
   print,'solution'
   print,rhs
   new(*,ix)=new(*,ix)+rhs
   str='' & read,str
endfor
endelse
;
bound,x,new
;
u=new
return 
end
