;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro tvdlf2,x,u,dt,t,pdelta=pdelta
@common
;   
; TVDLF method
;
;  Toth section 4.2
;
a=size(u)
ny=a(1) & nx=a(2)
;
; uniform grid
;
dx=x(1)-x(0)
;
; delta ubar 4.16 of Toth
;
dub=delubar(u,limiter=param.limiter)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  TVDLF loop, if changes big then try again w much smaller dt
;    which is guaranteed to make a small correction.  This is essentially
;    just a time step limiter to ensure the preddictor/corrector 
;    corrections are small. 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; predictor step eq. 4.11
;
xtra=call_function(param.driver,u,x,t)
dd=(flux(x,u+0.5*dub)-flux(x,u-0.5*dub))/dx -(source(u) + xtra)
unphalf= u  - 0.5 * dt * dd 
;
; check change magnitudes, reduce dt as needed
;
pdelta=changes(unphalf,u)  ; actual changes 
ratio= (1./pdelta)/param.slowdown < 1.d0  ; allowed/actual
dt = (( dt*ratio ) > param.tsmall) < .1d0
if(dt le 0.d0) then message,'dt too small'
;
; proceed with smaller step
;
unphalf=u -0.5* dt * dd
;
;  linear extrapolants, eqs 4.12, must use 0.5*dx/dxav
;
sh=[0,1]
left=unphalf + 0.5*dub 
right=shift(unphalf-0.5*dub,-sh)
;
; fluxes at cell interfaces, eqns. 4.9 4.10
;
fp= (flux(x,left)+flux(x,right))*.5d0
fm=shift(fp,sh)
;
; non-linear form
;
; propagation speed, arithmetic mean
;
av=(unphalf+shift(unphalf,-sh))/2.
cmax=u*0.
c=csound(av) ;+ abs(av(1,*)/av(0,*))
; orig for i=0,ny-1 do cmax(i,*)=c
for i=1,2 do cmax(i,*)=c   ; p, E
cmax(0,*)=abs(u(1,*)/u(0,*))
for i=3,ny-1 do cmax(i,*)=cmax(0,*)   
; 
phip=dt/dx*cmax*(right-left)
phim=shift(phip,sh)
;
;corrector  eq. 4.3
;
corre = -dt/dx*(fp-fm) + 0.5*(phip-phim)
;
; implicit or explicit
;
xtra=call_function(param.driver,unphalf,x,t)
if(param.implicit gt 0.d0) then begin 
   indx=[0,1,2,3,4]  ;  equations treated implicitly
   corri=impstep(x,u,dt,corre,indx) 
   new=u + source(unphalf)*dt + xtra*dt
   for i=0,ny-1 do begin 
      fe=1.d0 & fi=0.d0
      k=where(indx eq i,nimp)
      if(nimp ne 0) then begin 
         fi=param.implicit
         fe=(1.-fi)
      endif
      new[i,*]= new[i,*] + corre[i,*]*fe + corri[i,*]*fi
   endfor
endif else begin 
   new=u + corre + (source(unphalf)+xtra)*dt ; explicit case
endelse
;
bound,x,new
;
u=new
return 
end
