pro postproc,lines,tmax=tmax,el=el,sp=sp,equilibrium=equilibrium,lrange=lrange
;+
;  post-processes data to produce total emission line emissivities 
;  erg/cm2/s/sr 
;  the structure lines is created and 
;  appended with emission rate coefficients
;-
@common
f='ok'
type='timedep'
if(n_elements(equilibrium) ne 0) then type='equilibrium'
;
restore,'u0.sav' ; get param structure from first time step
t=0l
iel=atomn(el,/num)
;lines=linedata(el=iel,/reset,omega_min=1.e-5) ; get emission line atomic data parameters
lines=linedata(omega_min=8.e-6) ; get emission line atomic data parameters

str1='all'
if(n_elements(el) ne 0) then begin
   k=where(lines.el eq el,c)
   if(c eq 0) then message,'no lines found for this element '+atomn(el,/str)
   lines=lines(k)
   str1=atomn(el,/str)
endif

str2='all'
if(n_elements(sp) ne 0) then begin
   k=where(lines.sp eq sp,c)
   if(c eq 0) then message,'no lines found for this spectrum '+string(sp)
   lines=lines(k)
   str2=roman(sp)
endif

k=where(lines.lambda gt lrange(0) and lines.lambda lt lrange(1)) & lines=lines(k)
nl=n_elements(lines)
print,nl,' lines used '

ifile='u'+strtrim(string(t),2)+'.sav'
f=findfile(ifile)
if(f ne '') then restore,f(0)
nx=n_elements(x)
;
j=findfile('u[0-9]*.sav')
if(n_elements(tmax) ne 0) then nt=tmax else nt = n_elements(j)
;
; define emission structure
emiss=fltarr(nx,nt)
emiss={emiss:emiss, pos:x, readme:'emiss(position along loop,time) erg/cm3/s/sr ',type:type}
emiss=replicate(emiss,nl)
lines=add_tag(lines,emiss,'emiss')
;
;
const=param.bconst/param.echarge
;
eold=0
print,'postproc: storing coefficients in lines.emiss for '+str1+' '+str2+'...'
for t=0,nt-1 do begin 
   ifile='u'+strtrim(string(t),2)+'.sav'
   f=findfile(ifile)
   if(f ne '') then restore,f(0)
   if (n_elements(equilibrium) ne 0) then begin
      se,u,e
      u=e
   endif
   if(t mod 50 eq 0) then print,'doing time step ',t,' of ',nt
;
; thermodynamics
;
   v=pt(u)
   temp=reform(v(1,*))
   p=reform(v(0,*))
   nne=nelec(u)
   velkm=reform(u(1,*)/u(0,*)/1.d5)
   for il=0,n_elements(lines)-1 do begin 
      ili=lines(il)
      e_per_ion= 8.63e-06*param.echarge*ili.e*ili.omega/(temp^ili.p)/ $
                 exp(ili.e/const/temp) 
;
; get population density of ions
;
      elem=ili.el
      sp=ili.sp
      if(elem ne eold) then spec=specno(elem,desc,indx)
      ok=where(spec eq sp)
      iond=reform(u(indx(ok),*))  ; ion density cm-3
      eold=elem
;
; get line emission rate erg/cm3/s/sr 
;
      lines(il).emiss.emiss(*,t)= iond*e_per_ion*nne/4./!pi
   endfor
endfor
print,'storing emission coefficients in lines.emiss... done'
;
return
end
