pro postprof,lines
;+
;  uses data from postproc to get line profiles
;  appends array isynthetic(wvelength,position on sun, time)
;  and doppler to lines
;-
@common
f='ok'
t=0l
;
a=size(lines.emiss.emiss) & nl=n_elements(lines) & nx=a(1) & nt=a(2)
n=n_elements(lines)
;
print,'reading u*sav..'
for ii=0,nt-1 do begin 
   t=ii
   ifile='u'+strtrim(string(t),2)+'.sav'
   f=findfile(ifile)
   if(f ne '') then restore,f(0)
   nx=n_elements(x)
   if(ii eq 0) then begin 
      vel=fltarr(nx,nt)
      te=vel
; cosine   
      xmax=max(x) & xmin=min(x)
      xx= (x-xmin)/(xmax-xmin)*!dpi
      costh= cos(xx)
      yy=(1-costh)*xmax/!dpi  ; this is the horizontal position as seen vertically on
   endif
   vel(*,ii)=u(1,*)/u(0,*)  *costh
   te(*,ii)=(pt(u))[1,*]
endfor
print,'reading.. done'
;
ww=-150+findgen(61)*5 ; doppler units km/s
ww=-200+findgen(50)*8 
ww*=1.e5 ; cm/s
nw=n_elements(ww)
;
profil=fltarr(nw,nx,nt)
;
;!p.multi=[0,8,nl,0,0]
;!p.charsize=1.6
;!x.margin=[6,1]
;!y.margin=[4,1]
turbw=10.e5
;
; grid to integrate emissivities
;
ymn=min(yy) & ymx=max(yy)
print,'ymx=ymn',ymx-ymn
ny=round((ymx-ymn)/.725e8*5)  ; .2" boxes
;ny=round((ymx-ymn)/.725e8*1)  ; 1" boxes
y=ymn+findgen(ny)/(ny-1)*(ymx-ymn)

dy=y(1)-y(0)

ngbyte= float(nl)*float(nw)*float(ny)*float(nt)*4/1.e9
str=strcompress(string(nl) + ','+string(nw)+','+string(ny) + ','+string(nt),/rem)
print,'output array will be ('+str+'), total = ',ngbyte,'  GByte'
if(ngbyte gt 1.) then begin 
   print,'enter .continue<return>  to proceed'
   stop
endif

isynthetic=fltarr(nw,ny,nt)
synth={i:isynthetic, dopp:ww, pos:y, readme:'synth.i= intensity (wavelength, position, time) erg/cm2/s/sr'}
dx=(x(1)-x(0))*abs(costh)       ; path length

synth=replicate(synth,nl)

lines=add_tag(lines,synth,'synth')

for il=0,n_elements(lines)-1 do begin 
   thermw= sqrt(param.bconst*te/param.umass/(2*lines(il).el))
   totw=sqrt(turbw^2+thermw^2)
   print,'synthetic profiles.. for line',il
   for iw=0,nw-1 do begin 
      xs= (ww(iw)+vel)/totw
;
; normalization is int(x,P(x))=1.0, P(x)=exp(-x^2)/sqrt(pi)
; normalization for constant doppler width (not x) integrations, dx/dw
; needed so divide by totw. Then int w,P(w) dw is correct
;
      profil[iw,*,*]=  lines(il).emiss.emiss * exp( -xs*xs)/ sqrt(!pi)/totw ;gaussian, normalized correctly
   endfor
   print,'synthetic profiles..done'
;
; 
   goto,skipem
   ist=roman(max(lines[il].sp))
   title=strcompress(atomn(lines[il].el,/str)) +' '+ist+' '+string(lines[il].lambda,format='(f11.1)')+' '+angstrom()
   print,title
   for pos=28,28+7*6,6 do begin 
      xtitle='Doppler shift km/s' & ytitle='time s'
      titlep=''
      if(il eq 1) then titlep= strcompress(' x='+string(x(pos)/1.e8,form='(f7.2)'))+' Mm'

      if(il ne (n_elements(lines)-1) or pos ne 28) then begin 
         xtitle=''
         ytitle=''
      endif
      plot_image,reform(profil[*,pos,*] ),scale=[(ww(1)-ww(0))/1.e5,1],origin=[min(ww)/1.e5,1],$
                 xtitle=xtitle,ytitle=ytitle,title=titlep,/nosq
      if(pos eq 28) then xyouts,min(ww)/1.e5+10,240,/data,strcompress(title),charsize=0.8

   endfor ; position
   skipem:
;
;  integrate over paths at each position
;
   for iy=0,ny-1 do begin
      ok=where(yy gt y(iy)-dy/2 and yy lt y(iy)+dy/2,nok)
      if(nok gt 0) then begin 
         for it=0,nt-1 do lines(il).synth.i(*,iy,it)=total(profil[*,ok,it],2)*dx
      endif
   endfor
endfor 

return
end
