function specno, element, desc,index, reset=reset
;+
;  returns the indices of ions (1=neutral, 2=+, 3=++ etc)
;  for element from the array desc.
;
;  inputs:
;    element  - string (e.g. 'He')
;    desc - description of the states solved
;  output:
;    returns the ions belonging to the element (e.g. [1,2] for H)
;    index (the indices of desc corresponding to the element selected)
;
;  use /reset on first call to reset/ initialize data
;  this version stores the data afer a reset call and is much
;  faster, it is called every timestep. 
;
; modified 27 Feb 2013 PGJ: bug found on indexing of arrays
;-
;
common specnos,pspec
;
if(n_elements(reset) ne 0) then delvarx,pspec
n=n_elements(desc)
nn=30
;
; first call
;
if(n_elements(pspec) eq 0) then begin 
   pspec={el:0,ions:intarr(nn),indx:intarr(nn),nions:0}
   pspec=replicate(pspec,nn)
endif
;
;  search 
;
elem=atomn(element,/num)
k=where(pspec.el eq elem,nk)   ; is element stored already? 
if(nk eq 0) then begin   ; no then build pspec array 
   ionss=-1
   index=-1
   for i=3,n-1 do begin 
      el=atomn(getwrd(desc[i]),/num)
      if(el eq elem) then begin
         ion=getwrd(desc[i],1)
         if(ion eq '') then begin 
            ionss=[ionss,0]
         endif else begin 
            j=getwrd(ion,delim='+')
            ionss=[ionss,fix(j)]
         endelse
         index=[index,i]
      endif
   endfor
   if(n_elements(index) eq 1) then return,-1    ; no match found, bug fix
   q=where(index ge 0)
   index=index[q]
   q=where(ionss ge 0)
   spnos=ionss[q]+1
   nq=n_elements(spnos)
   pspec[elem].el=elem
   pspec[elem].ions[0:nq-1]=spnos
   pspec[elem].indx[0:nq-1]=index
   pspec[elem].nions=nq
endif
;
; return data
;
nq=pspec[elem].nions
index=(pspec[elem].indx)[0:nq-1]
return,(pspec[elem].ions)[0:nq-1]
end
